#include <Servo.h>

// Déclaration des broches
const int trigPin = 9; // Broche TRIG connectée à D9
const int echoPin = 10; // Broche ECHO connectée à D10
const int distanceSeuil = 5; // Distance seuil en cm
const int tempsLimite = 8000; // Temps limite pour 3 détections (en millisecondes)
const int tempsSansDetectionSeuil = 5000; // Temps avant de réinitialiser

Servo monServo; // Création de l'objet Servo

int compteurPassages = 0; // Compteur de passages sous le seuil
unsigned long dernierPassage = 0; // Enregistrement du dernier passage
unsigned long debutFenetre = 0; // Début de la fenêtre de temps pour les détections
unsigned long dernierTempsSansDetection = 0;
bool objetPresent = false;

void setup() {
  pinMode(trigPin, OUTPUT); // Configurer TRIG comme sortie
  pinMode(echoPin, INPUT);  // Configurer ECHO comme entrée
  Serial.begin(9600);       // Initialisation de la communication série

  monServo.attach(11);      // Attache le servomoteur à la broche 11
  monServo.write(25);        // Initialise le servomoteur à l'angle de repos
}

void loop() {
  // Envoi d'un signal TRIG
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  // Lecture de la durée de l'écho
  long duration = pulseIn(echoPin, HIGH);

  // Calcul de la distance (en cm)
  long distance = duration * 0.034 / 2;

  Serial.print("Distance : ");
  Serial.print(distance);
  Serial.println(" cm");

  unsigned long tempsActuel = millis();

  // Vérifier si la distance est inférieure ou égale au seuil
  if (distance <= distanceSeuil) {
    if (compteurPassages == 0) {
      debutFenetre = tempsActuel; // Initialiser la fenêtre de temps
    }
    compteurPassages++;
    dernierPassage = tempsActuel;
    objetPresent = true;
    Serial.println("Passage détecté sous le seuil!");
  } else {
    if (objetPresent) {
      objetPresent = false;
      dernierTempsSansDetection = tempsActuel;
    }
  }

  // Vérifier si 3 passages ont été détectés dans la fenêtre de 5 secondes
  if (compteurPassages >= 3 && (tempsActuel - debutFenetre <= tempsLimite)) {
    Serial.println("Condition remplie : Activation du servomoteur!");
    monServo.write(60); // Activer le servomoteur
    delay(4000);        // Maintenir la position pendant 4 secondes
    monServo.write(25);  // Revenir à l'angle de repos
    compteurPassages = 0; // Réinitialiser le compteur
  }

  // Réinitialiser si le temps limite est dépassé sans atteindre 3 passages
  if (tempsActuel - dernierPassage > tempsLimite) {
    compteurPassages = 0;
  }

  // Réinitialiser si aucune détection n'est faite après un certain temps
  if (!objetPresent && (tempsActuel - dernierTempsSansDetection > tempsSansDetectionSeuil)) {
    Serial.println("Aucune détection prolongée, réinitialisation du compteur.");
    compteurPassages = 0;
  }

  delay(350); // Petite pause pour éviter une boucle trop rapide
}
